@@ -21,13 +21,14 @@ Concepts:
     a projects is just a program/package
 """
 
-import os
-import sys
 import argparse
 import asyncio
+import os
+import sys
+
+from .builder import Build
 from .exceptions import eprint
 from .warner import ZCBEWarner
-from .builder import Build
 
 # All available types of warnings (gcc-like)
 ALL_WARNINGS = {
@@ -80,7 +81,7 @@ def start():
     # This has to be a internal class as it uses warner
     class WarningsAction(argparse.Action):
         # pylint: disable=too-few-public-methods
-        """Argparse action to modify warning behaviour."""
+        """Argparse action to modify warning behavior."""
 
         def __init__(self, option_strings, dest, nargs=1, **kwargs):
             super().__init__(option_strings, dest, nargs, **kwargs)
@@ -108,7 +109,7 @@ def start():
     parser.add_argument("-w", help="Suppress all warnings",
                         action=WarningsAction, nargs=0)
     parser.add_argument("-W", metavar="WARNING",
-                        help="Modify warning behaviour", action=WarningsAction)
+                        help="Modify warning behavior", action=WarningsAction)
     parser.add_argument("-B", "--rebuild", action="store_true",
                         help="Force build requested projects and dependencies")
     parser.add_argument("-C", "--chdir", type=str, help="Change directory to")
@@ -118,6 +119,8 @@ def start():
                         help="Build all projects in mapping.toml")
     parser.add_argument("-s", "--silent", action="store_true",
                         help="Silence make standard output")
+    parser.add_argument("-n", "--dry-run", action="store_true",
+                        help="Don't actually run any commands")
     parser.add_argument("-H", "--about", type=str, action=AboutAction,
                         help='Help on a topic("topics" for a list of topics)')
     parser.add_argument('projects', metavar='PROJ', nargs='*',
@@ -128,6 +131,7 @@ def start():
     # Create builder instance
     builder = Build(".", warner, if_silent=namespace.silent,
                     if_rebuild=namespace.rebuild,
+                    if_dryrun=namespace.dry_run,
                     build_toml_filename=namespace.file)
     if namespace.all:
         runner = builder.build_all()
